@@ -21,9 +21,13 @@
 #include "config.h"
 
 #include <string>
+#include <exception>
 #include <QApplication>
 #include <QLocale>
 #include <QTranslator>
+#include "logging.h"
+#include "base_exception.h"
+#include "stacktrace.h"
 #include "main_window.h"
 #ifdef WIN32
 #   include <windows.h>
@@ -32,11 +36,47 @@
 #   include <CoreFoundation/CoreFoundation.h>
 #endif
 
-using std::string;
+#ifdef HAVE_EXECINFO_H
+#   include <signal.h>
+#   include <execinfo.h>
+#endif
 
+using namespace std;
+
+
+static void bt_sighandler(int);
+
+// Custom QApplication to catch and log exceptions properly
+// See http://forum.qtfr.org/viewtopic.php?id=7615
+class MyApplication : public QApplication
+{
+public:
+    MyApplication(int argc, char **argv)
+        : QApplication(argc, argv)
+    {}
+
+    virtual bool notify(QObject *receiver, QEvent *event)
+    {
+        try
+        {
+            return QApplication::notify(receiver, event);
+        }
+        catch (const BaseException &e)
+        {
+            LOG_ROOT_ERROR("Exception caught: " << e.what() << "\n" << e.getStackTrace());
+            return false;
+        }
+    }
+};
 
 int main(int argc, char **argv)
 {
+#ifdef HAVE_EXECINFO_H
+    // Install a custom signal handler to print a backtrace when crashing
+    // See http://www.linuxjournal.com/article/6391 for inspiration
+    signal(SIGSEGV, &bt_sighandler);
+#endif
+
     // On Mac, running Eliot from the dock does not automatically set the LANG
     // variable, so we do it ourselves.
     // Note: The following block of code is copied from VLC, and slightly
@@ -45,7 +85,7 @@ int main(int argc, char **argv)
     /* Check if $LANG is set. */
     if (NULL == getenv("LANG"))
     {
-        // Retrieve the preferred language as chosen in  System Preferences.app
+        // Retrieve the preferred language as chosen in System Preferences.app
         // (note that CFLocaleCopyCurrent() is not used because it returns the
         // preferred locale not language)
         CFArrayRef all_locales = CFLocaleCopyAvailableLocaleIdentifiers();
@@ -77,7 +117,7 @@ int main(int argc, char **argv)
 #endif
 #endif
 
-    QApplication app(argc, argv);
+    MyApplication app(argc, argv);
     app.setWindowIcon(QIcon(":/images/eliot.xpm"));
 
 #ifdef ENABLE_NLS
@@ -121,3 +161,16 @@ int main(int argc, char **argv)
     qmain.show();
     return app.exec();
 }
+
+#ifdef HAVE_EXECINFO_H
+static void bt_sighandler(int signum)
+{
+    LOG_ROOT_FATAL("Segmentation fault!");
+    LOG_ROOT_FATAL("Backtrace:\n" << StackTrace::GetStack());
+
+    // Restore the default handler to generate a nice core dump
+    signal(signum, SIG_DFL);
+    raise(signum);
+}
+#endif
+
