@@ -20,7 +20,7 @@
 
 #include "config.h"
 
-#include <iostream>
+//#include <iostream>
 #include <QtGui/QLabel>
 #include <QtGui/QMessageBox>
 #include <QtGui/QFileDialog>
@@ -29,7 +29,9 @@
 #include <QtGui/QPrintDialog>
 #include <QtGui/QPrinter>
 #include <QtGui/QPainter>
+#include <QtGui/QDesktopServices>
 #include <QtCore/QSettings>
+#include <QtCore/QUrl>
 
 #include "main_window.h"
 #include "dic.h"
@@ -60,6 +62,8 @@
 #include "coord.h"
 
 
+INIT_LOGGER(qt, MainWindow);
+
 const char *MainWindow::m_windowName = "MainWindow";
 
 MainWindow::MainWindow(QWidget *iParent)
@@ -67,6 +71,12 @@ MainWindow::MainWindow(QWidget *iParent)
     m_prefsDialog(NULL), m_bagWindow(NULL), m_boardWindow(NULL),
     m_historyWindow(NULL), m_dicToolsWindow(NULL), m_dicNameLabel(NULL)
 {
+#ifdef DEBUG
+    // Check that the string conversion routines are not buggy
+    checkConversions();
+#endif
+
+    LOG_DEBUG("Creating main window");
     m_ui.setupUi(this);
     createMenu();
     readSettings();
@@ -79,10 +89,9 @@ MainWindow::MainWindow(QWidget *iParent)
     // but we would lose the desktop integration...
     unsigned int val = time(NULL);
     srand(val);
-#ifdef DEBUG
+
     // Make it easier to reproduce bugs
-    cout << "Rand seed: " << val << endl;
-#endif
+    LOG_DEBUG("Rand seed: " << val);
 
     QObject::connect(this, SIGNAL(gameChanged(const PublicGame*)),
                      this, SLOT(updateForGame(const PublicGame*)));
@@ -125,6 +134,8 @@ MainWindow::MainWindow(QWidget *iParent)
                      historyTab, SLOT(setGame(const PublicGame*)));
     QObject::connect(this, SIGNAL(gameUpdated()),
                      historyTab, SLOT(refresh()));
+    QObject::connect(historyTab, SIGNAL(requestDefinition(QString)),
+                     this, SLOT(showDefinition(QString)));
     QHBoxLayout *hlayout2 = new QHBoxLayout;
     hlayout2->addWidget(historyTab);
     m_ui.groupBoxHistory->setLayout(hlayout2);
@@ -143,8 +154,8 @@ MainWindow::MainWindow(QWidget *iParent)
     QObject::connect(m_playersWidget, SIGNAL(gameUpdated()), this, SIGNAL(gameUpdated()));
     QObject::connect(m_playersWidget, SIGNAL(notifyProblem(QString)),
                      this, SLOT(displayErrorMsg(QString)));
-    QObject::connect(m_playersWidget, SIGNAL(notifyInfo(QString)),
-                     this, SLOT(displayInfoMsg(QString)));
+    QObject::connect(m_playersWidget, SIGNAL(requestDefinition(QString)),
+                     this, SLOT(showDefinition(QString)));
 
     // Players score
     ScoreWidget *scores = new ScoreWidget;
@@ -162,9 +173,10 @@ MainWindow::MainWindow(QWidget *iParent)
     QString dicPath = qs.value(PrefsDialog::kINTF_DIC_PATH, "").toString();
     if (dicPath != "")
     {
+        LOG_INFO("Using dictionary " << lfq(dicPath));
         try
         {
-            m_dic = new Dictionary(qtl(dicPath));
+            m_dic = new Dictionary(lfq(dicPath));
         }
         catch (DicException &e)
         {
@@ -193,6 +205,8 @@ void MainWindow::destroyCurrentGame()
     if (m_game == NULL)
         return;
 
+    LOG_DEBUG("Destroying current game");
+
     // Some controls, like the board, can live when there is no game.
     // We only have to give them a NULL handler instead of the current one.
     emit gameChangedNonConst(NULL);
@@ -202,6 +216,8 @@ void MainWindow::destroyCurrentGame()
 
     delete m_game;
     m_game = NULL;
+
+    LOG_DEBUG("Game destroyed");
 }
 
 
@@ -216,9 +232,10 @@ void MainWindow::refresh()
         m_actionHistoryNextTurn->setEnabled(!isLastTurn);
         m_actionHistoryLastTurn->setEnabled(!isLastTurn);
         m_actionHistoryReplayTurn->setEnabled(!isLastTurn);
+        if (m_game->isFinished())
+            displayInfoMsg(_q("End of the game"));
 #ifdef DEBUG
-        cout << endl << endl;
-        m_game->printTurns();
+        //m_game->printTurns();
 #endif
     }
 }
@@ -226,6 +243,7 @@ void MainWindow::refresh()
 
 void MainWindow::prefsUpdated()
 {
+    LOG_DEBUG("Preferences updated");
     // Disconnect the training rack updates from the "Plus 1" tab of the
     // dictionary tools
     m_playersWidget->disconnect(SIGNAL(trainingRackUpdated(const QString&)));
@@ -295,6 +313,7 @@ void MainWindow::updateStatusBar(const Dictionary *iDic)
 
 void MainWindow::displayErrorMsg(QString iMsg, QString iContext)
 {
+    LOG_ERROR("Displayed error: " << lfq(iMsg));
     if (iContext == "")
         iContext = _q("Eliot - Error");
 
@@ -304,10 +323,33 @@ void MainWindow::displayErrorMsg(QString iMsg, QString iContext)
 
 void MainWindow::displayInfoMsg(QString iMsg)
 {
+    LOG_INFO("Displayed message: " << lfq(iMsg));
     statusBar()->showMessage(iMsg);
 }
 
 
+void MainWindow::showDefinition(QString iWord)
+{
+    QSettings qs(ORGANIZATION, PACKAGE_NAME);
+    QString url = qs.value(PrefsDialog::kINTF_DEFINITIONS_SITE_URL,
+                           PrefsDialog::kDEFAULT_DEF_SITE).toString();
+    if (url == "")
+    {
+        displayErrorMsg(_q("No definitions site defined.\n"
+                           "Please define one in the preferences."));
+        return;
+    }
+
+    url = url.replace("%w", iWord.toLower());
+    url = url.replace("%W", iWord.toUpper());
+    bool res = QDesktopServices::openUrl(QUrl(url));
+    if (!res)
+    {
+        LOG_ERROR("Could not open URL: " << lfq(url));
+    }
+}
+
+
 void MainWindow::closeEvent(QCloseEvent *event)
 {
     // Make sure auxiliary windows don't survive after the main one
@@ -360,21 +402,17 @@ void MainWindow::changeDictionary(QString iFileName)
         if (m_game)
         {
             QString msg = _q("Loading a dictionary will stop the current game.");
-            QMessageBox confirmationBox(QMessageBox::Question, _q("Eliot"), msg,
-                                        QMessageBox::Yes | QMessageBox::No, this);
-            confirmationBox.setInformativeText(_q("Do you want to continue?"));
-            confirmationBox.setDefaultButton(QMessageBox::Yes);
-            confirmationBox.setEscapeButton(QMessageBox::No);
-            int res = confirmationBox.exec();
-            if (res == QMessageBox::No)
+            if (!requestConfirmation(msg))
                 return;
         }
 
+        LOG_INFO("Loading new dictionary file: " << lfq(iFileName));
+
         destroyCurrentGame();
 
         try
         {
-            Dictionary *dic = new Dictionary(qtl(iFileName));
+            Dictionary *dic = new Dictionary(lfq(iFileName));
             delete m_dic;
             m_dic = dic;
             emit dicChanged(m_dic);
@@ -392,6 +430,18 @@ void MainWindow::changeDictionary(QString iFileName)
 }
 
 
+bool MainWindow::requestConfirmation(QString msg)
+{
+    QMessageBox confirmationBox(QMessageBox::Question, _q("Eliot"), msg,
+                                QMessageBox::Yes | QMessageBox::No, this);
+    confirmationBox.setInformativeText(_q("Do you want to continue?"));
+    confirmationBox.setDefaultButton(QMessageBox::Yes);
+    confirmationBox.setEscapeButton(QMessageBox::No);
+    int res = confirmationBox.exec();
+    return res == QMessageBox::Yes;
+}
+
+
 QAction * MainWindow::addMenuAction(QMenu *menu, QString iText,
                                     const QKeySequence &iShortcut,
                                     QString iStatusTip, const char *iMember,
@@ -431,7 +481,7 @@ void MainWindow::createMenu()
                   false, QIcon(":/images/printer.png"));
     menuFile->addSeparator();
     addMenuAction(menuFile, _q("&Quit"), _q("Ctrl+Q"),
-                  _q("Quit Eliot"), SLOT(close()),
+                  _q("Quit Eliot"), SLOT(onGameQuit()),
                   false, QIcon(":/images/quit_16px.png"));
 
     QMenu *menuHistory = new QMenu(m_ui.menubar);
@@ -499,6 +549,8 @@ void MainWindow::createMenu()
 
 void MainWindow::onGameNew()
 {
+    LOG_DEBUG("Starting a new game (unconfirmed)");
+
     if (m_dic == NULL)
     {
         displayErrorMsg(_q("You have to select a dictionary (.dawg file) "
@@ -515,6 +567,13 @@ void MainWindow::onGameNew()
     if (res == QDialog::Rejected)
         return;
 
+    if (m_game)
+    {
+        QString msg = _q("Starting a new game will stop the current one.");
+        if (!requestConfirmation(msg))
+            return;
+    }
+
     // Destroy the game and the associated controls
     destroyCurrentGame();
 
@@ -525,11 +584,11 @@ void MainWindow::onGameNew()
 
     m_ui.groupBoxPlayers->show();
 
+    displayInfoMsg(_q("Game started"));
     m_game->start();
     emit gameChangedNonConst(m_game);
     emit gameChanged(m_game);
     emit gameUpdated();
-    displayInfoMsg(_q("Game started"));
 }
 
 
@@ -546,7 +605,7 @@ void MainWindow::onGameLoad()
     {
         try
         {
-            PublicGame *tmpGame = PublicGame::load(qtl(fileName), *m_dic);
+            PublicGame *tmpGame = PublicGame::load(lfq(fileName), *m_dic);
             destroyCurrentGame();
             m_game = tmpGame;
         }
@@ -574,7 +633,7 @@ void MainWindow::onGameSaveAs()
     {
         try
         {
-            m_game->save(qtl(fileName));
+            m_game->save(lfq(fileName));
             displayInfoMsg(_q("Game saved"));
         }
         catch (std::exception &e)
@@ -587,6 +646,7 @@ void MainWindow::onGameSaveAs()
 
 void MainWindow::onGamePrint()
 {
+    LOG_DEBUG("Printing game (unconfirmed)");
     if (m_game == NULL)
         return;
 
@@ -594,6 +654,8 @@ void MainWindow::onGamePrint()
     QPrintDialog printDialog(&printer, this);
     if (printDialog.exec() == QDialog::Accepted)
     {
+        LOG_INFO("Printing game");
+
         QPainter painter(&printer);
         const History &history = m_game->getHistory();
 
@@ -727,10 +789,19 @@ void MainWindow::onGamePrint()
         // Total score
         nextHeight += LINE_HEIGHT;
         painter.drawText(curWidth, nextHeight, QString("%1").arg(score));
+
+        LOG_INFO("Game printed");
     }
 }
 
 
+void MainWindow::onGameQuit()
+{
+    LOG_INFO("Exiting");
+    close();
+}
+
+
 void MainWindow::onSettingsPreferences()
 {
     if (m_prefsDialog == NULL)
@@ -838,6 +909,8 @@ void MainWindow::onWindowsDicTools()
                                     m_actionWindowsDicTools);
         QObject::connect(this, SIGNAL(dicChanged(const Dictionary*)),
                          dicTools, SLOT(setDic(const Dictionary*)));
+        QObject::connect(dicTools, SIGNAL(requestDefinition(QString)),
+                         this, SLOT(showDefinition(QString)));
         // Link the training rack with the "Plus 1" one
         QSettings qs(ORGANIZATION, PACKAGE_NAME);
         if (qs.value(PrefsDialog::kINTF_LINK_TRAINING_7P1, false).toBool())
@@ -859,7 +932,7 @@ void MainWindow::onHelpAbout()
     QString msg;
     msg.sprintf("Eliot %s\n\n", VERSION);
     msg += _q( \
-        "Copyright (C) 1999-2010 - Antoine Fraboulet & Olivier Teuliere\n\n" \
+        "Copyright (C) 1999-2011 - Antoine Fraboulet & Olivier Teuliere\n\n" \
         "This program is free software; you can redistribute it and/or " \
         "modify it under the terms of the GNU General Public License as " \
         "published by the Free Software Foundation; either version 2 of " \
@@ -938,13 +1011,7 @@ void MainWindow::onHistoryReplayTurn()
         QString msg = _q("Replaying this turn will modify the game history "
                          "by deleting the turns after the displayed one (i.e. "
                          "turns \"in the future\").");
-        QMessageBox confirmationBox(QMessageBox::Question, _q("Eliot"), msg,
-                                    QMessageBox::Yes | QMessageBox::No, this);
-        confirmationBox.setInformativeText(_q("Do you want to continue?"));
-        confirmationBox.setDefaultButton(QMessageBox::Yes);
-        confirmationBox.setEscapeButton(QMessageBox::No);
-        int ret = confirmationBox.exec();
-        if (ret != QMessageBox::Yes)
+        if (!requestConfirmation(msg))
             return;
     }
 
