@@ -55,11 +55,12 @@ public:
 
 
 PlayWordMediator::PlayWordMediator(QObject *parent, QLineEdit &iEditPlay,
-                                   QLineEdit &iEditCoord, QPushButton &iButtonPlay,
+                                   QLineEdit &iEditCoord, QLineEdit &iEditPoints,
+                                   QPushButton &iButtonPlay,
                                    CoordModel &iCoordModel, PublicGame *iGame)
     : QObject(parent), m_game(iGame), m_lineEditPlay(iEditPlay),
-    m_lineEditCoord(iEditCoord), m_pushButtonPlay(iButtonPlay),
-    m_coordModel(iCoordModel)
+    m_lineEditCoord(iEditCoord), m_lineEditPoints(iEditPoints),
+    m_pushButtonPlay(iButtonPlay), m_coordModel(iCoordModel)
 {
     m_lineEditPlay.setFocus();
     // These strings cannot be in the .ui file, because of the newlines
@@ -89,15 +90,31 @@ PlayWordMediator::PlayWordMediator(QObject *parent, QLineEdit &iEditPlay,
                      this, SLOT(lineEditCoord_returnPressed()));
     QObject::connect(&m_pushButtonPlay, SIGNAL(clicked()),
                      this, SLOT(pushButtonPlay_clicked()));
-    QObject::connect(&m_coordModel, SIGNAL(coordChanged(const Coord&)),
-                     this, SLOT(updateCoord(const Coord&)));
+    QObject::connect(&m_coordModel, SIGNAL(coordChanged(const Coord&, const Coord&)),
+                     this, SLOT(updateCoord(const Coord&, const Coord&)));
 }
 
 
 void PlayWordMediator::lineEditPlay_textChanged()
 {
-    m_pushButtonPlay.setEnabled(m_lineEditPlay.hasAcceptableInput() &&
-                                m_lineEditCoord.hasAcceptableInput());
+    bool acceptableInput =
+        m_lineEditPlay.hasAcceptableInput() &&
+        m_lineEditCoord.hasAcceptableInput();
+    m_pushButtonPlay.setEnabled(acceptableInput);
+
+    if (!acceptableInput)
+        m_lineEditPoints.clear();
+    else
+    {
+        // Compute the points of the word
+        const wstring &word = getWord();
+        const wstring &coords = wfq(m_lineEditCoord.text());
+        int points = m_game->computePoints(word, coords);
+        if (points >= 0)
+            m_lineEditPoints.setText(QString("%1").arg(points));
+        else
+            m_lineEditPoints.setText("#");
+    }
 }
 
 
@@ -107,8 +124,69 @@ void PlayWordMediator::lineEditPlay_returnPressed()
         !m_lineEditCoord.hasAcceptableInput())
         return;
 
+    const wstring &word = getWord(true);
+    QString coords = m_lineEditCoord.text();
+    int res = m_game->play(word, wfq(coords));
+    if (res == 0)
+    {
+        emit gameUpdated();
+        m_lineEditPlay.setFocus();
+    }
+    else
+    {
+        // Try to be as explicit as possible concerning the error
+        QString msg = _q("Cannot play '%1' at position '%2':\n%3")
+            .arg(m_lineEditPlay.text()).arg(coords);
+        switch (res)
+        {
+            case 1:
+                msg = msg.arg(_q("Some letters are not valid for the current dictionary"));
+                break;
+            case 2:
+                msg = msg.arg(_q("Invalid coordinates"));
+                break;
+            case 3:
+                msg = msg.arg(_q("The word does not exist"));
+                break;
+            case 4:
+                msg = msg.arg(_q("The rack doesn't contain the letters needed to play this word"));
+                break;
+            case 5:
+                msg = msg.arg(_q("The word is part of a longer one"));
+                break;
+            case 6:
+                msg = msg.arg(_q("The word tries to replace an existing letter"));
+                break;
+            case 7:
+                msg = msg.arg(_q("An orthogonal word is not valid"));
+                break;
+            case 8:
+                msg = msg.arg(_q("The word is already present on the board at these coordinates"));
+                break;
+            case 9:
+                msg = msg.arg(_q("A word cannot be isolated (not connected to the placed words)"));
+                break;
+            case 10:
+                msg = msg.arg(_q("The first word of the game must be horizontal"));
+                break;
+            case 11:
+                msg = msg.arg(_q("The first word of the game must cover the H8 square"));
+                break;
+            case 12:
+                msg = msg.arg(_q("The word is going out of the board"));
+                break;
+            default:
+                msg = msg.arg(_q("Incorrect or misplaced word"));
+        }
+        emit notifyProblem(msg);
+    }
+}
+
+
+wstring PlayWordMediator::getWord(bool emitSignal)
+{
     // Convert the jokers to lowercase
-    const wistring &inputWord = qtw(m_lineEditPlay.text().toUpper());
+    const wistring &inputWord = wfq(m_lineEditPlay.text().toUpper());
     // Convert to internal representation, then back to QString
     QString word = qfw(m_game->getDic().convertFromInput(inputWord));
 
@@ -116,12 +194,15 @@ void PlayWordMediator::lineEditPlay_returnPressed()
     while ((pos = word.indexOf('(')) != -1)
     {
         if (word.size() < pos + 3 || word[pos + 2] != ')' ||
-            !m_game->getDic().validateLetters(qtw(QString(word[pos + 1]))))
+            !m_game->getDic().validateLetters(wfq(QString(word[pos + 1]))))
         {
             // Bug in validate()!
             // This should never happen
-            QString msg = _q("Cannot play word: misplaced parentheses");
-            emit notifyProblem(msg);
+            if (emitSignal)
+            {
+                QString msg = _q("Cannot play word: misplaced parentheses");
+                emit notifyProblem(msg);
+            }
             break;
         }
         else
@@ -134,81 +215,26 @@ void PlayWordMediator::lineEditPlay_returnPressed()
 
     // Convert the input string into an internal one
     const wstring intWord =
-        m_game->getDic().convertFromInput(qtw(word));
+        m_game->getDic().convertFromInput(wfq(word));
 
-    QString coords = m_lineEditCoord.text();
-    int res = m_game->play(intWord, qtw(coords));
-    if (res == 0)
-    {
-        emit gameUpdated();
-        m_lineEditPlay.setFocus();
-    }
-    else
-    {
-        // Try to be as explicit as possible concerning the error
-        QString msg = _q("Cannot play '%1' at position '%2':\n")
-            .arg(m_lineEditPlay.text()).arg(coords);
-        switch (res)
-        {
-            case 1:
-                msg += _q("Some letters are not valid for the current dictionary");
-                break;
-            case 2:
-                msg += _q("Invalid coordinates");
-                break;
-            case 3:
-                msg += _q("The word does not exist");
-                break;
-            case 4:
-                msg += _q("The rack doesn't contain the letters needed to play this word");
-                break;
-            case 5:
-                msg += _q("The word is part of a longer one");
-                break;
-            case 6:
-                msg += _q("The word tries to replace an existing letter");
-                break;
-            case 7:
-                msg += _q("An orthogonal word is not valid");
-                break;
-            case 8:
-                msg += _q("The word is already present on the board at these coordinates");
-                break;
-            case 9:
-                msg += _q("A word cannot be isolated (not connected to the placed words)");
-                break;
-            case 10:
-                msg += _q("The first word of the game must be horizontal");
-                break;
-            case 11:
-                msg += _q("The first word of the game must cover the H8 square");
-                break;
-            case 12:
-                msg += _q("The word is going out of the board");
-                break;
-            default:
-                msg += _q("Incorrect or misplaced word (%1)").arg(1);
-        }
-        // FIXME: the error is too generic
-        emit notifyProblem(msg);
-    }
+    return intWord;
 }
 
 
 void PlayWordMediator::lineEditCoord_textChanged(const QString &iText)
 {
-    Coord c(qtw(iText));
+    Coord c(wfq(iText));
     if (!(m_coordModel.getCoord() == c))
-        m_coordModel.setCoord(Coord(qtw(iText)));
+        m_coordModel.setCoord(Coord(wfq(iText)));
     lineEditPlay_textChanged();
 }
 
 
-void PlayWordMediator::updateCoord(const Coord &iCoord)
+void PlayWordMediator::updateCoord(const Coord &, const Coord &iNewCoord)
 {
-    if (iCoord.isValid() && m_lineEditCoord.text() != qfw(iCoord.toString()))
-        m_lineEditCoord.setText(qfw(iCoord.toString()));
-    else if (!iCoord.isValid() && m_lineEditCoord.hasAcceptableInput())
+    if (iNewCoord.isValid() && m_lineEditCoord.text() != qfw(iNewCoord.toString()))
+        m_lineEditCoord.setText(qfw(iNewCoord.toString()));
+    else if (!iNewCoord.isValid() && m_lineEditCoord.hasAcceptableInput())
         m_lineEditCoord.setText("");
     lineEditPlay_textChanged();
 }
@@ -228,7 +254,7 @@ QValidator::State PlayWordValidator::validate(QString &input, int &) const
     if (input == "")
         return Intermediate;
 
-    const wistring &winput = qtw(input);
+    const wistring &winput = wfq(input);
     // The string is invalid if it contains invalid input characters
     if (!m_dic.validateInputChars(winput, L"()") || input.contains('?'))
         return Invalid;
@@ -246,7 +272,7 @@ QValidator::State PlayWordValidator::validate(QString &input, int &) const
     while ((pos = qintInput.indexOf('(')) != -1)
     {
         if (qintInput.size() < pos + 3 || qintInput[pos + 2] != ')' ||
-            !m_dic.validateLetters(qtw(QString(qintInput[pos + 1]))))
+            !m_dic.validateLetters(wfq(QString(qintInput[pos + 1]))))
         {
             return Intermediate;
         }
@@ -272,13 +298,13 @@ CoordsValidator::CoordsValidator(QObject *parent)
 QValidator::State CoordsValidator::validate(QString &input, int &) const
 {
     // Only authorize characters part of a valid coordinate
-    wstring copy = qtw(input.toUpper());
+    wstring copy = wfq(input.toUpper());
     wstring authorized = L"ABCDEFGHIJKLMNO1234567890";
     if (copy.find_first_not_of(authorized) != wstring::npos)
         return Invalid;
 
     // Check coordinates
-    Coord c(qtw(input));
+    Coord c(wfq(input));
     if (!c.isValid())
         return Intermediate;
 
