@@ -61,7 +61,8 @@ PlayerWidget::PlayerWidget(QWidget *parent, CoordModel &iCoordModel,
 
     // Use the mediator
     m_mediator = new PlayWordMediator(this, *lineEditPlay, *lineEditCoords,
-                                      *pushButtonPlay, iCoordModel, iGame);
+                                      *lineEditPoints, *pushButtonPlay,
+                                      iCoordModel, iGame);
     QObject::connect(m_mediator, SIGNAL(gameUpdated()),
                      this, SIGNAL(gameUpdated()));
     QObject::connect(m_mediator, SIGNAL(notifyProblem(QString)),
@@ -140,31 +141,56 @@ void PlayerWidget::on_lineEditChange_textChanged()
 {
     pushButtonChange->setEnabled(lineEditChange->hasAcceptableInput() &&
                                  lineEditChange->text() != "");
-    pushButtonPass->setEnabled(lineEditChange->text() == "");
+    // Force the letters to be in upper case
+    lineEditChange->setText(lineEditChange->text().toUpper());
 }
 
 
 void PlayerWidget::on_lineEditChange_returnPressed()
 {
     ASSERT(m_game->getMode() == PublicGame::kFREEGAME,
-           "Trying to pass or change letters while not in free game mode");
+           "Trying to change letters while not in free game mode");
 
-    QString inputLetters = lineEditChange->text();
+    pass(lineEditChange->text());
+}
+
+
+void PlayerWidget::on_pushButtonPass_clicked()
+{
+    ASSERT(m_game->getMode() == PublicGame::kFREEGAME,
+           "Trying to pass while not in free game mode");
+
+    pass("");
+}
+
+
+void PlayerWidget::pass(QString inputLetters)
+{
     // Convert the input string into an internal one
     const wstring &letters =
-        m_game->getDic().convertFromInput(qtw(inputLetters));
+        m_game->getDic().convertFromInput(wfq(inputLetters));
+
     // Pass the turn (and possibly change letters)
     int res = m_game->freeGamePass(letters);
     if (res == 0)
         emit gameUpdated();
     else
     {
-        // FIXME: the error is too generic
         QString msg;
         if (inputLetters == "")
-            msg = _q("Cannot pass turn (%1)").arg(res);
+            msg = _q("Cannot pass turn:\n%1");
         else
-            msg = _q("Cannot change letters '%1' (%2)").arg(inputLetters).arg(res);
+            msg = _q("Cannot change letters '%1':\n%2").arg(inputLetters);
+        if (res == 1)
+            msg = msg.arg(_q("Changing letters is not allowed when there are less than 7 tiles left in the bag"));
+        else if (res == 2)
+            msg = msg.arg(_q("The rack of the current player does not contain all the listed letters"));
+        else if (res == 3)
+            msg = msg.arg(_q("The game is already finished!"));
+        else if (res == 3)
+            msg = msg.arg(_q("Some letters are invalid for the current dictionary"));
+        else
+            msg = msg.arg(_q("Unknown error"));
         emit notifyProblem(msg);
     }
 }
@@ -182,7 +208,7 @@ ChangeValidator::ChangeValidator(QObject *parent,
 QValidator::State ChangeValidator::validate(QString &input, int &) const
 {
     // The string is invalid if it contains invalid input characters
-    const wistring &winput = qtw(input);
+    const wistring &winput = wfq(input);
     if (!m_dic.validateInputChars(winput))
         return Invalid;
 
@@ -193,7 +219,7 @@ QValidator::State ChangeValidator::validate(QString &input, int &) const
     if (!m_dic.validateLetters(intInput))
         return Intermediate;
 
-    const wstring &rack = m_dic.convertFromInput(qtw(m_lineEdit.text()));
+    const wstring &rack = m_dic.convertFromInput(wfq(m_lineEdit.text()));
     if (intInput.size() > rack.size())
         return Intermediate;
     // The letters to change must be in the rack
@@ -255,6 +281,8 @@ void PlayerTabWidget::setGame(PublicGame *iGame)
                              this, SIGNAL(gameUpdated()));
             QObject::connect(trWidget, SIGNAL(rackUpdated(const QString&)),
                              this, SIGNAL(trainingRackUpdated(const QString&)));
+            QObject::connect(trWidget, SIGNAL(requestDefinition(QString)),
+                             this, SIGNAL(requestDefinition(QString)));
             addTab(trWidget, qfw(player.getName()));
         }
         else
